pathDir='/Users/Bernd/Werkbank/SciLab/Ballistics';
getd(pathDir+'/utilities');

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// WGS-84 ellipsoid

a=6378137.000;
b=6356752.314;
eps=1-b/a; // epsilon = Flattening
e=sqrt(1-b^2/a^2);
eP2=e^2;
e2=e^2/(2-e^2);

// ω (radians/s) = earth's angular spin rate
// The defining value for the World Geodetic System 1984 (WGS 84)
// ellipsoid is

omega=7.292115E-5;

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Radius of earth

// φ = geographic latitude
// φ' = geocentric latitude
// Re = Radius of the earth at geographic latitude φ

function rRe=fRe(Phi)
    deltaPhi=-e2*sin(2*Phi);

    rRe=a*sqrt((1-eP2)/(1-eP2*(cos(Phi+deltaPhi))^2));
endfunction

RadarData.Re=fRe(RadarData.phi1);

// h1 (m) = altitude of the radar above mean sea level
// rv(u,v,w) (m) = location vector of the projectile,
//     coordinates are (u,v,w) relative to an ENU system at the radar.
// Res = radius of a spherical earth
// h (m) = altitude of the projectile above mean sea level

ReS=fRe(%pi/4);

function rh=fh(h,rV)
    u=rV(1);
    v=rV(2);
    w=rV(3);
    rVp2=u*u+v*v+w*w;

    rh=(2*ReS*(w+h1)+h1*(2*w+h1)+rVp2)/(sqrt((ReS+w+h1)^2+rVp2)+ReS);
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//  Gravity acceleration at the location of the projectile

// Radar data
// φ1 = geodedic (geographic) latitude
// h1 = altitude (meters)

// A = coefficient of principal latitude term
// B = coefficient of secondary latitude term
// C = coefficient of altitude term
// g45 = gravity acceleration at 45° latitude (sea level)

// g1 = gravity acceleration at latitude φ1 and altitude h1

function rg1=fg1(phi1,h1)
    g45=9.806154;
    A=-0.025928;
    B=0.000069;
    C=-0.000003;

    rg1=g45+A*cos(2*phi1)+B*(cos(2*phi1))^2+C*h1
endfunction

RadarData.gB=fg1(RadarData.phi1,RadarData.h1);

// h = projectile altitude
// g = gravity acceleration at the projectile altitude h

function rg=fg(phi1,h1,h) // Not used for the ballistics
    Re=fRe(phi1);
    g1=fg1(phi1,h1);

    rg=g1*(1-2*(h-h1)/Re+(h-h1)*(3*h-h1)/Re^2);
endfunction

// These equations only account for the difference in height between the
// radar and the projectile. The difference in latitude is ignored since
// it is a very small effect.

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Vector of gravity acceleration at altitude h

// Radar data
// phi1 = geodedic (geographic) latitude
// h1 = altitude (meters)

// Projectile data
// x = position vector
// rbg = gravity acceleration vector

function rbG=fbG(RadarD,ProjD,x)
    Re=RadarData.Re;
    gB=RadarData.gB;

    OmegaBO=ProjD.OmegaBO;

    t1=ProjD.FiringPosition(1);
    t2=ProjD.FiringPosition(2);
    t3=ProjD.FiringPosition(3);

    x1=x(1);
    x2=x(2);
    x3=x(3);

    gBtRe=gB/Re;
    // gBtRe=0;

    rbG(1)=-gBtRe*(x1+t1*cos(OmegaBO)+t2*sin(OmegaBO));
    rbG(2)=-gBtRe*(x2-t1*sin(OmegaBO)+t2*cos(OmegaBO));
    rbG(3)=-gB+2*gBtRe*(x3+t3);

//    rbG(1)=0;
//    rbG(2)=0;
//    rbG(3)=-9.80616; // g45
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rbGfree=fbGfree()
    rbGfree(1)=0;
    rbGfree(2)=0;
    rbGfree(3)=-9.80616; // g45
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Vector of coriolis acceleration

// Radar data
// phi1 = geodedic (geographic) latitude

// Projectile data
// x = position vector
// rbg = gravity acceleration vector

function rbC=fbC(RadarD,ProjD,v)
    phi1=RadarD.phi1;
    OmegaBO=ProjD.OmegaBO;

    v1=v(1);
    v2=v(2);
    v3=v(3);

    rbC(1)= 2*omega*(-v2*sin(phi1)             +v3*cos(phi1)*cos(OmegaBO));
    rbC(2)= 2*omega*( v1*sin(phi1)             -v3*cos(phi1)*sin(OmegaBO));
    rbC(3)=-2*omega*(-v1*cos(phi1)*cos(OmegaBO)+v2*cos(phi1)*sin(OmegaBO));

//    rbC(1)=0;
//    rbC(2)=0;
//    rbC(3)=0;
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// ρ0 (kg/m³) = the air density at sea level
// Ha (m) = air density scale heigh,
//    the altitude at which the air density has a value of 1/e of its
//    value at sea level
// ρ (kg/m³) = the air density at altitude h

rho0=1.223;
Ha=9335.4;

function rrho=frho(h)
        rrho=rho0*exp(-h/Ha);
endfunction

function rrhoDash=frhoDash(h)
    rrhoDash=-frho(h)/Ha;
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// a0 (m/s) = speed of sound at sea level (h = 0)
// a (m/s) = speed of sound at altitude h
// Hs (m) = speed of sound scale height

a0=340.6;
Hs=44445.0;

function ra=fa(h)
    if h<Hs then
       ra=a0*sqrt(1-h/Hs);
    else
       ra=0;
    end
endfunction

function raDash=faDash(h) // ### TO_BE_DONE ###
    raDash=a0^2/(2*fa(h)*Hs);
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Polynomial coefficients for the universal drag model
// Velocity Interval
// (Mach No.)- Polynomial Coefficients--------------------------------------------
// From  To    C0          C1            C2            C3            C4

UDM_Data=..
  [0.000 0.820 0.050668862 0.0013913995  0.00031414626 0.00078841815 -0.0093582755
   0.820 0.920 57.110734   -278.34028    509.16192     -413.95175    126.19562
   0.920 0.960 -855.61088  3690.0146     -5960.9818    4274.5582     -1147.856
   0.960 1.020 707.87922   -2824.5275    4217.9902     -2794.1724    692.95925
   1.020 1.150 -111.06631  387.02358     -504.87914    292.67035     -63.619847
   1.150 1.250 0.17435318  0.099544718   -0.19561716   0.10018737    -0.017651053
   1.250 3.000 0.20694309  -0.0040499634 -0.071698139  0.034380058   -0.0046420421];

// Number of mach number intervals in the table

[MI_Nb,nc]=size(UDM_Data);

// Mach number interval data

MID=UDM_Data(:,1:2);

// Greatest Mach value in the table

MGr=MID(MI_Nb,2);

// Polynomial Coefficients
C0=UDM_Data(:,3);
C1=UDM_Data(:,4);
C2=UDM_Data(:,5);
C3=UDM_Data(:,6);
C4=UDM_Data(:,7);

// Mach number M
// a = speed of sound

function rM=fM(V)
    rM=abs(V)/fa(V);
endfunction

// Mach number zone
// M = 1 .. MI_Nb+1;

function rMZ=fMZ(M)
    MZ=MI_Nb+1;

    if M>=MID(MI_Nb,2) then
        rMZ=MZ;
        return;
    else
        MZ=MZ-1;
    end

    while M<MID(MZ,1)
        MZ=MZ-1;
    end

    rMZ=MZ;
endfunction

// Universal drag in interval zones
// MI = Mach number interval, 1 .. MI_Nb
// M = Mach number in interval MI
// MID(MI,1)<=M<MID(MI,2)

function rCd1=fCd1(MI,M)
    rCd1=C0(MI)+..
         C1(MI)*M+..
         C2(MI)*M^2+..
         C3(MI)*M^3+..
         C4(MI)*M^4;
endfunction

// Drag value of greatest Mach in table

CdGr=fCd1(MI_Nb,MGr);

// Universal aerodynamic drag for M>=MG

function rCd2=fCd2(M)
    rCd2=CdGr*sqrt(MGr/M);
endfunction

// Universal drag for velocity V

function rCd=fCd(V)
    M=fM(V);
    MZ=fMZ(M);

    if MZ==MI_Nb+1 then
        rCd=fCd2(M);
    else
        rCd=fCd1(MZ,M);
    end

endfunction

// Universal aerodynamic drag for velocity V - rocket impact

function rCd_RI=fCd_RI(V)
    rCD_RI=max(0.1,fCd(V));
endfunction

// Accelaration due to atmospheric drag
// avDrag = acceleration vector due to atmospheric drag
// ρ (kg/m³) = air density
// Kd (m²/kg) = drag parameter, it depends on the projectile's shape and mass
// Cd = the dimensionless aerodynamic drag coefficient
// rvDot (m/s) = velocity vector
// h (m) = altitude of the projectile

function ravDrag=favDrag(Kd,rvDot,h,Wv,OmegaBO)

    Wv1= Wv(1)*cos(OmegaBO)+Wv(2)*sin(OmegaBO);
    Wv2=-Wv(1)*sin(OmegaBO)+Wv(2)*cos(OmegaBO);
    Wv3=-Wv(3);

    u=rvDot(1)-Wv1;
    v=rvDot(2)-Wv2;
    w=rvDot(3)-Wv3;

    V=sqrt(u^2+v^2+w^2);
    M=fM(V);

    ravDrag=-frho(h)*Kd*fCd(V)*V*rvDot;
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rbD=fbD(RadarD,ProjD,x,v)
    Wv=RadarD.Wv;
    Kd=ProjD.Kd;
    OmegaBO=ProjD.OmegaBO;

    h=x(3);
    rvDot=v;

    rbD=favDrag(Kd,rvDot,h,Wv,OmegaBO);

    // rbD=zeros(3,1);
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rbZ=fbZ(RadarD,ProjD)
    phi1=RadarD.phi1;
    Re=RadarData.Re;
    h1=RadarD.h1;
    OmegaBO=ProjD.OmegaBO;

    C=0.5*omega^2*(Re+h1);

    rbZ(1)=C*sin(2*phi1)*sin(OmegaBO);
    rbZ(2)=C*sin(2*phi1)*cos(OmegaBO);
    rbZ(3)=0;

//    rbZ(1)=0;
//    rbZ(2)=0;
//    rbZ(3)=0;
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rbS=fbS(ProjD,x,v)
    withSpin=ProjD.withSpin;
    lambda=ProjD.lambda;

    if (withSpin==%T) then
        rbS(1)=0;
        rbS(2)=-lambda;
        rbS(3)=0;
    else
        rbS=zeros(3,1);
    end
endfunction

// #####################################################################
// #####################################################################

function rvDot=fvDot(x,v)
    // return ph data

    rvDot(1:3)=v;
    rvDot(4:6)=fbG(RadarData,ProjectileData,x)+..
               fbD(RadarData,ProjectileData,x,v)+..
               fbC(RadarData,ProjectileData,v)+..
               fbS(ProjectileData,x,v)+..
               fbZ(RadarData,ProjectileData);
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rvDotfree=fvDotfree(x,v)
    // return ph data

    rvDotfree(1:3)=v;
    rvDotfree(4:6)=fbGfree();
endfunction

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rBallistic=fBallistic(t,ph)
    // returns 'phDot data'
    x=ph(1:3);
    v=ph(4:6);
    rBallistic=fvDot(x,v);
endfunction;

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function rBallisticfree=fBallisticfree(freet,ph)
    // returns 'phDot data'
    x=ph(1:3);
    v=ph(4:6);
    rBallisticfree=fvDotfree(x,v);
endfunction;

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ProjectileData.OmegaBO=fgetProjAngel(v_0);

function rRotOmegaBO=fRotOmegaBO(ProjD)
    OmegaBO=ProjD.OmegaBO;

    RotM=[ cos(OmegaBO)  sin(OmegaBO) 0
          -sin(OmegaBO)  cos(OmegaBO) 0
           0             0            1];

    rRotOmegaBO=RotM;
endfunction

RotOmegaBO=fRotOmegaBO(ProjectileData);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ph_0(1:3)=RotOmegaBO*(x_0-x_0);
ph_0(4:6)=RotOmegaBO*v_0;

tData=linspace(t_0,t_end,steps);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ng=1;

function [z]=g(t,ph)
    z=ph(3);
endfunction

[y,rd]=ode("roots",ph_0,t_0,tData,fBallistic,ng,g);

if (rd==[]) then
    disp("### Kein Nulldurchgang!");
    disp("### Integrationszeit t_end zu klein!");

    abort;
// else
    // mprintf('\n   Nulldurchgang  in s: %g\n', rd(1));
end

Delta=1; // 1%
t_end=rd(1)*(1+Delta/100);

tData=[];
tData=linspace(t_0,t_end,steps);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

phData=ode(ph_0,t_0,tData,fBallistic);

phDatafree=ode(ph_0,t_0,tData,fBallisticfree);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

[rIx,rP1,rP2]=fgetImpactInterval(phData);

to_deg=180/%pi;

if (rIx==0) then
    disp("### Kein Nulldurchgang!");
else

    // disp(rP1);
    // disp(rP2);

    tI1=fgetImpactTime(t_0,t_end,steps,rIx);
    tI2=fgetImpactTime(t_0,t_end,steps,rIx+1);

    [rt0,rP0]=fgetImpact(tI1,tI2,rP1,rP2);

    rP0_dcu=ffilt(rP0);

    rP0_enu=(RotOmegaBO'*rP0)+ProjectileData.FiringPosition;
    rP0_enu=ffilt(rP0_enu);

    [Zmax,Zindmax]=max(phData(3,:));

    mprintf('\nGeodätische Breite: %g\n',..
             to_deg*RadarData.phi1);

    mprintf('Startschwindigkeit: %g\n', mv);

    mprintf('Richtwinkel       : %g\n', qe_g);

    mprintf('Omega (Azimuth)   : %g\n',..
            ProjectileData.OmegaBO*to_deg);

    mprintf('Winddaten ENU     : %.1f | %.1f | %.1f\n',..
            RadarData.Wv(1), RadarData.Wv(2), RadarData.Wv(3));

    mprintf('===>\n');

    mprintf('größte Höhe       : %.1f\n', Zmax);

    mprintf('Aufschlagszeit    : %g\n', rt0);

    mprintf('Aufschlagsort DCU : %.1f | %.1f | %.1f\n',..
            rP0_dcu(1), rP0_dcu(2), rP0_dcu(3));

    mprintf('Aufschlagsort ENU : %.1f | %.1f | %.1f\n',..
            rP0_enu(1), rP0_enu(2), rP0_enu(3));
end

[Zmax,Zindmax]=max(phData(3,:));

maxHCoords=phData(1:3,Zindmax);

linesX(1,1)=maxHCoords(1);
linesX(2,1)=maxHCoords(1);
linesY(1,1)=maxHCoords(2);
linesY(2,1)=maxHCoords(2);
linesZ(1,1)=0;
linesZ(2,1)=maxHCoords(3);

linesX(1,2)=0;
linesX(2,2)=rP0(1);
linesY(1,2)=0;
linesY(2,2)=rP0(2);
linesZ(1,2)=0;
linesZ(2,2)=rP0(3);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

my_handle=scf(1);
clf(my_handle,"reset");

// disp(phData(3,:)');

a=gca();

drawlater();

xtitle ( " Flugbahn in DCU-Koordinaten " ,..
         " Entfernung [m] ", ..
         " " );

a.labels_font_size=3;
a.x_label.font_size = 3;
a.y_label.font_size = 3;
a.title.font_size = 3;

param3d1(phData(1,1:rIx+1)', phData(2,1:rIx+1)', phData(3,1:rIx+1)');

for Ix=1:rIx+1
    if (phDatafree(1,Ix) > phData(1,rIx+1)) then
        rIxfree=Ix+5;
        break;
    end
end

freeColor=[5];

param3d1(phDatafree(1,1:rIxfree)',..
         phDatafree(2,1:rIxfree)',..
         list(phDatafree(3,1:rIxfree)',freeColor)); // ,flag=[0,4]

freeCoords=phDatafree(1:3,rIxfree);

linesX(1,3)=freeCoords(1);
linesX(2,3)=freeCoords(1);
linesY(1,3)=freeCoords(2);
linesY(2,3)=freeCoords(2);
linesZ(1,3)=0;
linesZ(2,3)=freeCoords(3);

linesX(1,4)=0;
linesX(2,4)=freeCoords(1);
linesY(1,4)=0;
linesY(2,4)=freeCoords(2);
linesZ(1,4)=0;
linesZ(2,4)=0;

linesColors=[6;2;3;3];

param3d1(linesX,linesY,list(linesZ,linesColors));

drawnow();

// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

